
// On importe le module Foundation pour l'accès aux fonctions mathématiques utilisées dans random()
import Foundation

// On définit les deux nombre entre lesquels on va tirer aléatoirement
let ecartHaut = 100
let ecartBas = 0

// On stockera dans nombreInput le nombre entré par le joueur. On le déclare par défaut sur la valeur -1 pour ne pas qu'elle soit par défaut égale à nombreMystere et fausser la boucle de jeu
var nombreInput = -1

// nombreMystere contiendra le nombre à deviner
var nombreMystere = 0

// Ici le nombre de coups réalisé jusque là
var coups = 0

// On déclare les fonctions de génération de nombres aléatoires
func random() -> Float {
    /*
     La génération des nombres alétoires se fait par la fonction arc4random() contenu dans le module Foundation importé plus haut.
     Cette fonction va retourner un nombre aléatoire de type UInt32 c'est a dire un entier 32 bit compris entre 0 et 4294967295
     On converti ce nombre en type Float puis on le divise par OxFFFFFFFF (qui correspond à 4294967295 en hexadecimal) pour obtenir un nombre entre 0 et 1 que l'on retourne
 */
    return Float(arc4random()) / 0xFFFFFFFF
}

func random(min: Int, max: Int) -> Int {
    /*
     Ici, on apelle la fonction random() déclarée juste au dessus. On multiplie son résultat par floatAdd contenant l'écart entre min et max, puis on le décale de min.
     Par exemple, si random() retourne 0.5 et que min = 0 et max = 10,  on fait donc
     0.5 * (10 - 0) + 0 = 0.5 * 10 = 5
     On retorune ensuite le résulat arrondi à l'inférieur avec la fonction floor(), puis on retourne le résultat de type Int
 */
    let floatAdd = Float(max - min)
    return Int(floor(random() * floatAdd) + Float(min))
}

// Ici on déclare la fonction de démarrage du jeu
func setup() {
    // On tire le nombre à deviner
    nombreMystere = random(min:ecartHaut, max:ecartBas)
    
    // On remet à zéro le nombre entré par le joueur et le nombre de coups au cas où il ait choisi de relancer une partie
    nombreInput = -1
    coups = 0
    
    // On affiche le début du jeu
    print("Bienvenue !")
    print("Vous devez deviner un nombre entre \(ecartBas) et \(ecartHaut)")
    print(" ")
}

// Ici on déclare la fonction appelée lorsque le joueur a deviné le bon numéro
func foundNumber() {
    // On affiche le message de fin et la proposition pour rejouer
    print("Vous avez trouvé le nombre mystère (\(coups) coups)!")
    print("Rejouer (Y/N) ?")
    
    // On récupère l'entrée
    let replay = readLine()
    if replay == "Y" {
        // Si le joueur a entré "Y" on relance le jeu
        setup()
    }
}

// On lance le jeu
setup()

// On lance une boucle de jeu qui s'exécutera tant que le nombre entré est différent du nombre à deviner (c'est pour ça que l'on stocke -1 par défaut dans nombreInput)
while nombreInput != nombreMystere {
    // On récupère l'entrée du joueur
    let input = readLine()
    
    // On incrémente le nombre de coups de 1
    coups += 1
    
    // On teste si le nombre entré est un nombre de type Int (entier)
    if let nInput = Int(input!) {
        // On stocke le nombre entré par le joueur
        nombreInput = nInput
        
        if nInput == nombreMystere {
            // Si le nombre entré est égal au nombre à deviner, on lance la fonction de fin de jeu
            foundNumber()
        }
        else if nInput > nombreMystere {
            // Si le nombre est plus petit
            print("Le nombre est plus petit")
        }
        else if nInput < nombreMystere {
            // Si le nombre est plus grand
            print("Le nombre est plus grand")
        }
    }
    else {
        // Si  ce qu'a entré le joueur n'est pas un nombre
        print("Veuillez entrer un nombre !")
    }
}

